#include "ScanRequest.h"
#include "CalibrationController.h"
#include "RequestException.h"
#include "Sct/SctNames.h"
#include "Sct/BugReport.h"

namespace SctCalibrationController {
  namespace Ipc {
    
    MinimalScanRequest::MinimalScanRequest() :
      m_clockByTwo(false),
      m_width(768) {
    };
    
    CORBA::Boolean MinimalScanRequest::clockByTwoNice() {
      return m_clockByTwo;
    }
    
    CORBA::Long MinimalScanRequest::widthNice() {
      return m_width;
    }
    
    void MinimalScanRequest::setWidthNice(CORBA::Long value) {
      m_width=value;
    }
    
    void MinimalScanRequest::setClockByTwoNice(CORBA::Boolean value) {
      m_clockByTwo=value;
    }
    
    CORBA::Boolean MinimalScanRequest::configureModulesNice(){
      return false;
    }
    
    CORBA::Long MinimalScanRequest::delayNice(){
      return 1;
    }
    
    Sct_SctApi::Scan_ptr MinimalScanRequest::getScanNice(){
      //m_scan->incRef();
      return Sct_SctApi::Scan::_duplicate(m_scan);
    }
    
    void MinimalScanRequest::setScanNice(Sct_SctApi::Scan_ptr scan){
      m_scan=Sct_SctApi::Scan::_duplicate(scan);
    }
    
    CORBA::Boolean MinimalScanRequest::isRawNice(){
      return false;
    }



  ////////////////////////////////////
    
    ScanRequestImpl::ScanRequestImpl() : IPCObject<POA_Sct_CalibrationController::ScanRequest>() {
    }

    
    CORBA::Boolean ScanRequestImpl::isRawCorba(//Sct_CalibrationControllerStatus *_status
) {
      return isRawNice();
    }

  CORBA::Boolean ScanRequestImpl::clockByTwoCorba(//Sct_CalibrationControllerStatus *_status
                                             ) {
    return clockByTwoNice();
  }
    
    
    CORBA::Long ScanRequestImpl::widthCorba(//Sct_CalibrationControllerStatus *_status
) {
      return widthNice();
    }

    void ScanRequestImpl::setWidthCorba(//Sct_CalibrationControllerStatus *_status,
                                   CORBA::Long value) {
      setWidthNice(value);
    }
    
    void ScanRequestImpl::setClockByTwoCorba(//Sct_CalibrationControllerStatus *_status, 
                                        CORBA::Boolean value) {
      setClockByTwoNice(value);
    }
    
    CORBA::Boolean ScanRequestImpl:: configureModulesCorba(//Sct_CalibrationControllerStatus *_status
){
      return configureModulesNice();
    }
    
    CORBA::Long ScanRequestImpl::delayCorba(//Sct_CalibrationControllerStatus *_status
                              ){
      return delayNice();
    }
    
    Sct_SctApi::Scan_ptr ScanRequestImpl::getScanCorba(//Sct_CalibrationControllerStatus *_status
){
      return getScanNice();
    }
    
    void ScanRequestImpl::setScanCorba(//Sct_CalibrationControllerStatus *_status, 
                                  Sct_SctApi::Scan_ptr scan){
      Sct::BugReport("BUG01", "ScanRequest.cpp","Will this infinite recurse?  It must not! Sleeping 1 sec",1000000);
      setScanNice(scan);
    }


/*
ScanRequestAdapter::ScanRequestAdapter(Sct_CalibrationController::ScanRequest_ptr req) : request(req) {
}

bool ScanRequestAdapter::configureModules() {
  try {
    return request->configureModules();
  } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::configureModules failed", __FILE__, __LINE__);    
  };
}

bool ScanRequestAdapter::clockByTwo() {
  try {
    return request->clockByTwo();
  } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::clockByTwo failed", __FILE__, __LINE__);    
  };
}

void ScanRequestAdapter::setClockByTwo(bool value) {
  try {
    request->setClockByTwo(value);
  } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::setClockByTwo failed", __FILE__, __LINE__);    
  };
}

bool ScanRequestAdapter::isRaw() {
  try {
    return request->isRaw();
  } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::setConfigureModules failed", __FILE__, __LINE__);    
  };
}

long ScanRequestAdapter::delay() {
  try {
    return request->configureModules();
  } catch (const CORBA::Exception & status) {
    throw RequestException(status, "ScanRequestAdapter::setConfigureModules failed", __FILE__, __LINE__);    
  };
}

long ScanRequestAdapter::width() {
    try {
      return request->width();
    } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::setConfigureModules failed", __FILE__, __LINE__);    
    };
}

void ScanRequestAdapter::setWidth(long width) {
  try {
    request->setWidth(width);
  } catch (const CORBA::Exception & status) {
      throw RequestException(status, "ScanRequestAdapter::setConfigureModules failed", __FILE__, __LINE__);    
  };
}
    
Sct_SctApi::Scan_ptr ScanRequestAdapter::getScan() {
  try {
    return request->getScan();
  } catch (const CORBA::Exception & status) {
    throw RequestException(status, "ScanRequestAdapter::getScan failed", __FILE__, __LINE__);
  };
}

void ScanRequestAdapter::setScan(Sct_SctApi::Scan_ptr scan) {
  try {
    request->setScan(scan);
  } catch (const CORBA::Exception & status) {
	throw RequestException(status, "ScanRequestAdapter::setScan failed", __FILE__, __LINE__);
  };
}
*/
  }
}
