#ifndef QUICKTRIMRANGETEST_H
#define QUICKTRIMRANGETEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;
using namespace Sct;

namespace SctCalibrationController {
namespace Scripts {
    
class QuickTrimRangeTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static QuickTrimRangeTest tr;
	return tr._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr inapi, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(inapi);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "QuickTrimRangeTest::startTest can't set compression")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "QuickTrimRangeTest::startTest can't set edge detect")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "QuickTrimRangeTest::startTest can't set send mask")

        // QCAL (fC)
        APICALL(api, modifyABCDVar(ST_QCAL, 1), "QuickTrimRangeTest::startTest can't set qcal")

	name = "TrimRangeTest";
	testVariable=ST_TRIM_RANGE;
	for (unsigned int i=0; i<6; ++i) {
	    testPoints.push_back(getValue(i));
	}
	nScans = testPoints.size();
    }

    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	if (isFirstPoint(index)) {
            // set trim range
	    APICALL(api, modifyABCDVar(ST_TRIM_RANGE, getRange(index)), "QuickTrimRangeTest::getNextScan")
	}
         // set trim value
	APICALL(api, modifyABCDVar(ST_TRIM, getValue(index)), "QuickTrimRangeTest::getNextScan")

	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
	double offset = ranges[getRange(index)] * getValue(index);
	return sl->thresholdScanfCOffset(200, 1.0, offset);  // trim at 1 fC, but give it an offset
    }

    unsigned int getRange(unsigned int index) {
      return index<3 ? 0 : 1;
    }
    
    bool isFirstPoint(unsigned int index) {
      return index%3 == 0;
    }
    
    unsigned int getValue(unsigned int index) {
      switch (index%3) {
      case 0: return 1;
      case 1: return 6;
      case 2: return 15;
      }
    }
    
    virtual bool canFeedbackRealImpl() {
	return true;
    }    

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }
        
private:
    QuickTrimRangeTest() {}
    Sct_SctApi::SctApiIPC_var api;
    static double ranges[4];
};
}
}
#endif //TRIMRANGETEST_H
