#ifndef SCTCALIBRATIONCONTROLLER_RAWSCAN_H
#define SCTCALIBRATIONCONTROLLER_RAWSCAN_H

#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

  /** partially-configured Raw scan */
    
class RawScan : public ScanRequestImpl {
public:
  static ScanRequestImpl& rawScan(unsigned long nTrigs, bool issueSoftReset) {
    static RawScan request;
    static const long initialWidth = request.widthCorba();
    static bool first = true;
    if (first) {
      first=false;
    } else {
      request.setWidthCorba(initialWidth);
    };

    Sct_SctApi::SctApiIPC_var api = CalibrationControllerImpl::instance()->getApi();
    Sct_SctApi::Scan_var s;
    APIRETCALL(api, s, createScan(), "Failed to create scan")
    request.setScanNice(s);
    
    APICALL(s, setNTrigs(nTrigs), "Failed to set triggers")
    request.setClockByTwoNice(false); /// Important - set not in clock/2 mode by default!
    
    Sct_SctApi::Trigger_var t;
    APIRETCALL(s, t, getTrigger1(), "Failed to get Trigger1")

    APICALL(s, configure(1, 0, 2, 1), "Failed to set Scan information")

    if (issueSoftReset) {
      const int delayBetweenSoftResetAndTrigger = 30;
      APICALL(t, softL1A(delayBetweenSoftResetAndTrigger), "Failed to set trigger type with soft reset")      
    } else {
      APICALL(t, singleL1A(), "Failed to set trigger type")
    }

    return request;
  }

  virtual bool isRawNice(){ 
    return true;
  }

  virtual long delayNice(){
    return 1;
  }

  virtual bool configureModulesNice(){
    return true;
  }

private:
  RawScan(){;}
};
    
}
}

#endif //SCTCALIBRATIONCONTROLLER_RAWSCAN_H
