#ifndef SYNCHTRIGGERNOISETEST_H
#define SYNCHTRIGGERNOISETEST_H

#include <cmath>  // For fabs

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class SynchTriggerNoiseTest  : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
    static Sct_CalibrationController::TestRequest_ptr instance() {
      static SynchTriggerNoiseTest stnt;
      return stnt._this();
    }

    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = api;

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "SynchTriggerNoiseTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "SynchTriggerNoiseTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "SynchTriggerNoiseTest::startTest")

        // 1 fC threshold
        APICALL(api, modifyABCDVar(ST_QTHR, 1.0), "SynchTriggerNoiseTest::startTest")

	name = "SynchTriggerNoiseTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr request = sl->defaultScan();

	// Create scan
        Sct_SctApi::Scan_var s;
        APIRETCALL(api, s, createScan(), "SynchTriggerNoiseTest::getNextScan")
        request->setScanCorba(s);

        APICALL(s, setNTrigs(1000), "Failed to set triggers")

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(Sct_SctApi::Scan::distSlave, 0), "Failed to set distSlave option")

	APICALL(s, setOption(Sct_SctApi::Scan::tim, 1), "Failed to set TIM option")
	APICALL(s, setOption(Sct_SctApi::Scan::nth, 15), "Failed to set nth option")

	// Adjust trigger
        Sct_SctApi::Trigger_var t;
        APIRETCALL(s, t, getTrigger1(), "Failed to get trigger1")

        APICALL(t, singleL1A(), "Failed to set trigger type")

	// Configure scan
        APICALL(s, configure(ST_ELAPSED, 0, 5, 1), "Failed to set Scan information")

	return request;
    }

    virtual bool canFeedbackRealImpl() {
	return false;
    }

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    SynchTriggerNoiseTest() {}
    Sct_SctApi::SctApiIPC_var api;
};
    
}
}

#endif //SYNCHTRIGGERNOISETEST_H
