#include "Fitter.h"
#include "Sct/SctNames.h"
#include "Sct/IpcObjectException.h"
#include "FitterInterface.hh"
#include <mrs/message.h>
#include <ipc/core.h>
#include <pmg/pmg_initSync.h>


using namespace Sct;

/**
 * A main program to generate a FittingService executable.
 * @author Alan Barr, @date 20 Feb 2003.
 * <BR>
 */

int main(int argc, char** argv){
  using namespace SctFitter;
  using namespace std;

  setExceptionHandlers(argv[0]);
  
  IPCCore::init(argc, argv);

  try{
      FitterArguments opts(argc, argv);
      opts.print(std::cout);

      //Initialize the fitter with the default FitStrategy
      Fitter& f = Fitter::initialize(opts);

      if (!f.publish()){
	  throw IpcObjectException("FittingService failed to publish", __FILE__, __LINE__);
      }

      f.go();
      pmg_initSync();
      Fitter::getFitterServer().run();
  }
  catch (Throwable& e){
    std::cerr << e.what()<<std::endl;;
    e.sendToMrs(MRS_ERROR) ; 
    terminate(); 
  }
}
