#ifndef VCDRFGLKVBLFGDGHKJSDFGKJXCHVGKCVF
#define VCDRFGLKVBLFGDGHKJSDFGKJXCHVGKCVF

/// Use this deleter as the second constuctor argument to a boost::shared_ptr being used to hold onto IPCObjects which need to be deleted with "_destroy()" rather than "delete".

/**
 * @author Christopher Lester
 * Example
 *
 *      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName));
 *
 * which is the same as
 *
 *      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer( CONSTRUCTOR_ARGS ));
 *
 * where I have suppressed the irrelevant info.
 *
 */

#include "SctNames.h"

namespace Sct {


  template<typename T>
    class DestroyingDeleter {
    public:
    static boost::shared_ptr<T> shared_ptr(T * t) {
      return boost::shared_ptr<T>(t, DestroyingDeleter<T>() );
    };
    public:
    void operator() (T * t) throw() {  
      try {
	t->_destroy();
      } catch (...) {
	try {
	  MRSStream& msg = SctNames::Mrs();
	  msg << "DestroyingSharedPtr was trying to _destroy something, but unfortunaltely that _destroy method threw an exception.  We are in a method that is not permitted to throw exceptions, so we will now catch it, but THIS IS NOT SUPPOSED TO HAPPEN!  DO NOT TRUST further output! (And bear in mind that memory is probably now leaking!)" << ENDM ;
	} catch (...) {
	  // fall through
	};
	// fall through
      };
    };
  };
  
  
};

#endif
