#include "ISUtilities.h"
#include "SctNames.h"
#include "IoExceptions.h"

namespace Sct{

void ISUtilities::addOrUpdateOrThrow(std::string& name, ISInfo& info, const std::string& file, int line, severity sev){
  ISInfoDictionary& is = SctNames::getISDictionary();
  ISInfo::Status st=is.contains(name);
  if ( st==ISInfo::Success ) {
    st = is.update(name, info);
  } else if ( st==ISInfo::NotFound ) {
    st = is.insert(name, info);
  }
  if (st != ISInfo::Success) {
    throw IsException(st, "ISUtilities: can't update status", file, line);
  }
} 
  
void ISUtilities::addNoUpdateOrThrow(std::string& name, ISInfo& info, const std::string& file, int line, severity sev){
  ISInfoDictionary& is = SctNames::getISDictionary();
  ISInfo::Status st=is.insert(name, info);
  if (st != ISInfo::Success) {
    throw IsException(st, "ISUtilities can't update status", file, line);
  }
}

void ISUtilities::addOrUpdateOrMessage(std::string& name, ISInfo& info, const std::string& file, int line, severity sev){
  try {
    addOrUpdateOrThrow(name, info, file, line, sev);
  }catch(Throwable& e){
    e.sendToMrs(sev);
  }
}

void ISUtilities::addNoUpdateOrMessage(std::string& name, ISInfo& info, const std::string& file, int line, severity sev){
  try {
    addNoUpdateOrThrow(name, info, file, line, sev);
  }catch(Throwable& e){
    e.sendToMrs(sev);
  }
}

}

