#include "Configuration.h"
#include "ScanMap.h"
#include "Sct/Env.h"
#include "Sct/IoExceptions.h"
#include "Sct/OmniMarshalling.h"
#include "Sct_SctApi/SctApiServer.h"
#include "Sct/SctNames.h"

using namespace Sct;

namespace SctTest {

Configuration::Configuration() : IPCNamedObject<POA_SctTestApi::Configuration>(SctNames::getPartition(), SctTestApi::Configuration::instanceName){
    try {
	handy_setDataFile("${SCT_DAQ_ROOT}/installed/config/TestConfig/ModuleData.txt");
    } catch (Sct::FileException& fe) {
        fe.sendToMrs(MRS_INFORMATION);
    }
    handy_setFastTimes();
    //handy_setRealisticTimes();
}

Configuration& Configuration::instance() {
    static Configuration configuration;
    return configuration;
}

void Configuration::handy_setDataFile(string file) {
    string temp = Env::substituteVariables(file);
    ScanMap::instance().initialize(temp);
    dataFile = temp;
}

string Configuration::handy_getDataFile() const {
    return dataFile;
}

void Configuration::handy_setFastTimes() {
    handy_setScanTime(5);
    handy_setReadoutTime(1);
    handy_setMultiDspEfficiency(1);
}

void Configuration::handy_setRealisticTimes() {
    handy_setScanTime(30);
    handy_setReadoutTime(1);
    handy_setMultiDspEfficiency(0.9);
}

void Configuration::handy_setScanTime(unsigned int time) {
    scanTime = time;
}

unsigned int Configuration::handy_getScanTime() const {
    return scanTime;
}

void Configuration::handy_setMultiDspEfficiency(double efficiency) {
    dspEfficiency = efficiency;
}

double Configuration::handy_getMultiDspEfficiency() const {
    return dspEfficiency;
}

void Configuration::handy_setReadoutTime(unsigned int time) {
    readoutTime = time;
}

unsigned int Configuration::handy_getReadoutTime() const {
    return readoutTime;
}

//CORBA methods:
void Configuration::setDataFile (const char * file) {
    try {
        handy_setDataFile(file);
    } catch (Sct::FileException& e) {
        //_status->returnCode =
	 throw SctTestApi::NoSuchFileException();
    }
}

char * Configuration::getDataFile () {
    return copyStringToCorba(handy_getDataFile());
}

void Configuration::setFastTimes () {
    handy_setFastTimes();
}

void Configuration::setRealisticTimes () {
    handy_setRealisticTimes();
}

void Configuration::setScanTime (CORBA::Long time) {
    handy_setScanTime(time);
}

CORBA::Long Configuration::getScanTime () {
    return handy_getScanTime();
}

void Configuration::setMultiDspEfficiency (CORBA::Double efficiency) {
    handy_setMultiDspEfficiency(efficiency);
}

CORBA::Double Configuration::getMultiDspEfficiency () {
    return handy_getMultiDspEfficiency();
}

void Configuration::setReadoutTime (CORBA::Long time) {
    handy_setReadoutTime(time);
}

CORBA::Long Configuration::getReadoutTime () {
    return handy_getReadoutTime();
}

}
