#include <ipc/object.h>
#include <ipc/server.h>
#include <ipc/core.h>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include "SctApiScan_Test.h"
#include "../SctApiServer.h"
#include "SctApi/SctApi.h"
#include "sctConf/configXMLImpl.h"
#include <boost/test/unit_test.hpp>
#include <iostream>
#include "Sct/DestroyingDeleter.h"

//? Obsolete? std::auto_ptr<IPCServer> ipcServer;

using boost::unit_test_framework::test_suite;
using namespace boost::posix_time;
using namespace boost::gregorian;

namespace SctApi {
  namespace Tests {
    void testGetScan() {
      IPCPartition p("SCT");
      //? Obsolete? ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new ConfigurationXMLImpl)));

      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName/*, ipcServer.get()*/));

      Sct_SctApi::Scan_var newScan = api->createScan();

      BOOST_CHECK(!CORBA::is_nil(newScan)); 
    }

    void testGetScanTrigger() {
      IPCPartition p("SCT");
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new ConfigurationXMLImpl)));

      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName/*, ipcServer.get()*/));

      try {
	Sct_SctApi::Scan_var newScan = api->createScan();
	BOOST_CHECK(!CORBA::is_nil(newScan)); 
	
	try {
	  Sct_SctApi::Trigger_var newTrigger = newScan->getTrigger1();
	  BOOST_CHECK(!CORBA::is_nil(newTrigger)); 
	  
	  try {
	    newTrigger->singleL1A();
	  } catch(...) {
	    throw SctApiException("Bad return code from singleL1A");
	  }
	  
	} catch(...) {
	  throw SctApiException("Bad return code from getTrigger1 ");
	}
	
      } catch (...) {
        throw SctApiException("Bad return code from createScan ");
      }
    }

    void testGetDefaultScan() {
      IPCPartition p("SCT");
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new ConfigurationXMLImpl)));

      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName/*, ipcServer.get()*/));

      Sct_SctApi::Scan_var newScan = api->getDefaultScan();

      BOOST_CHECK(!CORBA::is_nil(newScan)); 
    }

    void testModifyDefaultScan() {
      IPCPartition p("SCT");
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new ConfigurationXMLImpl)));

      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName/*, ipcServer.get()*/));

      Sct_SctApi::Scan_var firstDefaultScan = api->getDefaultScan();
      BOOST_CHECK(!CORBA::is_nil(firstDefaultScan)); 
      firstDefaultScan->setOption(Sct_SctApi::Scan::debug, 15);

      Sct_SctApi::Scan_var secondDefaultScan = api->getDefaultScan();
      BOOST_CHECK(!CORBA::is_nil(secondDefaultScan)); 

      int defValue1 = firstDefaultScan->getOption(Sct_SctApi::Scan::debug);
      int defValue2 = secondDefaultScan->getOption(Sct_SctApi::Scan::debug);

      BOOST_CHECK(defValue1 == 15); 
      BOOST_CHECK(defValue2 == 15); 
    }

    void testCreateIsDefault() {
      IPCPartition p("SCT");
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi::SctApi::serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new ConfigurationXMLImpl)));

      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(*ob, Sct_SctApi::SctApiIPC::instanceName/*, ipcServer.get()*/));

      Sct_SctApi::Scan_var defScan = api->getDefaultScan();

      BOOST_CHECK(!CORBA::is_nil(defScan)); 

      defScan->setOption(Sct_SctApi::Scan::debug, 15);

      Sct_SctApi::Scan_var scan = api->createScan();

      BOOST_CHECK(!CORBA::is_nil(scan)); 

      int defValue = scan->getOption(Sct_SctApi::Scan::debug);

      BOOST_CHECK(defValue == 15); 
    }

    SctApiScan_Test::SctApiScan_Test() {
      int narg=0;
      char ** args=0;
      IPCCore::init(narg,args);
	add(BOOST_TEST_CASE(&testGetScan));
	add(BOOST_TEST_CASE(&testGetScanTrigger));
        add(BOOST_TEST_CASE(&testGetDefaultScan));
        add(BOOST_TEST_CASE(&testModifyDefaultScan));
        //        add(BOOST_TEST_CASE(&testCreateIsDefault));
    }	
}
}
