#include <iostream>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>
#include "Sct/Exception.h"
#include "configXMLImpl.h"
#include "boost/lexical_cast.hpp"
#include "boost/date_time/posix_time/posix_time.hpp"
#include <ipc/core.h>

using namespace std;
using namespace SctConfiguration;

boost::shared_ptr<SctConfiguration::Configuration> config;
list<string> printParameters;

string outfile="";

string row(string sn){
  unsigned mur, module, barrel, row;
  int number;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToBarrel(mur, module, barrel, row, number);
    return boost::lexical_cast<string>(row);
  }catch(exception&e){
    return "-";
  }
}

string number(string sn){
  unsigned mur, module, barrel, row;
  int number;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToBarrel(mur, module, barrel, row, number);
    return boost::lexical_cast<string>(number);
  }catch(exception&e){
    unsigned ring, num;
    int disk;
    try{
      config->translateToEndcap(mur, module, disk, ring, num);
      return boost::lexical_cast<string>(num);
    }catch(exception&e){
      return "-";
    }
  }
}

string barrel(string sn){
  unsigned mur, module, barrel, row;
  int number;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToBarrel(mur, module, barrel, row, number);
    return boost::lexical_cast<string>(barrel);
  }catch(exception&e){
    return "-";
  }
}

string disk(string sn){
  unsigned mur, module, ring, number;
  int disk;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToEndcap(mur, module, disk, ring, number);
    return boost::lexical_cast<string>(disk);
  }catch(exception&e){
    return "-";
  }
}

string ring(string sn){
  unsigned mur, module, ring, number;
  int disk;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToEndcap(mur, module, disk, ring, number);
    return boost::lexical_cast<string>(ring);
  }catch(exception&e){
    return "-";
  }
}

string mur(string sn){
  unsigned mur, module;
  try{
    config->translateFromSN(sn, mur, module);
    return boost::lexical_cast<string>(mur);
  }catch(exception&e){
    return "-";
  }
}


string pscrate(string sn){
  unsigned mur, module, partition, crate, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToPowerSupply(mur, module, partition, crate, channel);
    return boost::lexical_cast<string>(crate);
  }catch(exception&e){
    return "-";
  }
}

string pspartition(string sn){
  unsigned mur, module, partition, crate, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToPowerSupply(mur, module, partition, crate, channel);
    return boost::lexical_cast<string>(partition);
  }catch(exception&e){
    return "-";
  }
}

string pschannel(string sn){
  unsigned mur, module, partition, crate, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToPowerSupply(mur, module, partition, crate, channel);
    return boost::lexical_cast<string>(channel);
  }catch(exception&e){
    return "-";
  }
}

string partition(string sn){
  unsigned mur, module, partition, crate, rod, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToROD(mur, module, partition, crate, rod, channel);
    return boost::lexical_cast<string>(partition);
  }catch(exception&e){
    return "-";
  }
}

string crate(string sn){
  unsigned mur, module, partition, crate, rod, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToROD(mur, module, partition, crate, rod, channel);
    return boost::lexical_cast<string>(crate);
  }catch(exception&e){
    return "-";
  }
}

string rod(string sn){
  unsigned mur, module, partition, crate, rod, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToROD(mur, module, partition, crate, rod, channel);
    return boost::lexical_cast<string>(rod);
  }catch(exception&e){
    return "-";
  }
}

string channel(string sn){
  unsigned mur, module, partition, crate, rod, channel;
  try{
    config->translateFromSN(sn, mur, module);
    config->translateToROD(mur, module, partition, crate, rod, channel);
    return boost::lexical_cast<string>(channel);
  }catch(exception&e){
    return "-";
  }
}

string group(string sn){
  try{
    return boost::lexical_cast<string>( config->getModuleGroup(sn) );
  }catch(exception&e){
    return "-";
  }
}


void parseArgs(int argc, char** argv){
  list<string> args;
  for (int iarg=1; iarg<argc; ++iarg){
    args.push_back(argv[iarg]);
  }
  for (list<string>::const_iterator iarg=args.begin(); 
       iarg!=args.end(); ++iarg){
    if (*iarg=="-output"){
      if (++iarg!=args.end()) {
	outfile=*iarg;
      }
    } else {
      printParameters.push_back(*iarg);
    }
  }
}

void printOutput(std::ostream& os){
  using namespace boost;
  using namespace boost::posix_time;
  using namespace boost::gregorian;

  shared_ptr<ConfigurationXMLImpl> xml = dynamic_pointer_cast<ConfigurationXMLImpl> (config);
  if (xml.get()){
    os << "#FILE PARSED: " << xml->getConfigurationFileName() << endl;
  }
  ptime now = second_clock::local_time();
  os << "#DATE/TIME PARSED: " << now << endl;
  os << "#";
  for (list<string>::const_iterator ipar=printParameters.begin();
	 ipar!=printParameters.end(); ++ipar){
    os << *ipar << "\t";
  }
  os << endl;
  list<string> modules=config->listAllModules();
  for (list<string>::const_iterator imod=modules.begin(); imod!=modules.end(); ++imod){
    for (list<string>::const_iterator ipar=printParameters.begin();
	 ipar!=printParameters.end(); ++ipar){
      if (*ipar=="sn"){
	os << *imod << "\t";
      } else if (*ipar=="row"){
	os << row(*imod) << "\t";
      }else if (*ipar=="number"){
	os << number(*imod) << "\t";
      }else if (*ipar=="barrel"){
	os << barrel(*imod) << "\t";
      }else if (*ipar=="ring"){
	os << ring(*imod) << "\t";
      }else if (*ipar=="disk"){
	os << disk(*imod) << "\t";
      }else if (*ipar=="mur"){
	os << mur(*imod) << "\t";
      }else if (*ipar=="pscrate"){
	os << pscrate(*imod) << "\t";
      }else if (*ipar=="pschannel"){
	os << pschannel(*imod) << "\t";
      }else if (*ipar=="partition"){
	os << partition(*imod) << "\t";
      }else if (*ipar=="rod"){
	os << rod(*imod) << "\t";
      }else if (*ipar=="crate"){
	os << crate(*imod) << "\t";
      }else if (*ipar=="channel"){
	os << channel(*imod) << "\t";
      }else if (*ipar=="group"){
	os << group(*imod) << "\t";
      }else{
	os << "-\t";
      }
    }
    os << endl;
  }
}

void printHelp(){
  std::cerr << "Usage: geometryDump <-output file> <parameter list>" << endl;
  std::cerr << "Parameters: " << endl;
  std::cerr << "sn             : module serial number" << endl;
  std::cerr << "barrel         : barrel number (barrel)" << endl;
  std::cerr << "row            : row on barrel (barrel)" << endl;
  std::cerr << "disk           : disk number (endcap)" << endl;
  std::cerr << "ring           : ring on disk (endcap)" << endl;
  std::cerr << "number         : number on row or ring" << endl;
  std::cerr << "mur            : minimal unit of readout number" << endl;
  std::cerr << "pscrate        : power supply crate number" << endl;
  std::cerr << "pschannel      : power supply channel number" << endl;
  std::cerr << "partition      : DAQ partion" << endl;
  std::cerr << "crate          : DAQ crate in partition" << endl;
  std::cerr << "rod            : ROD in DAQ crate" << endl;
  std::cerr << "channel        : ROD channel" << endl;
  std::cerr << "group          : DAQ group" << endl;
}

int main(int argc, char** argv) {
  IPCCore::init(argc, argv);
  Sct::setExceptionHandlers("geometryDump");
  config.reset(new ConfigurationXMLImpl);
  try {
    parseArgs(argc, argv);
    if (printParameters.empty()){
      printHelp();
      return 1;
    }
    if (outfile==""){
      printOutput(cout);
    }else{
      std::cerr << "geometryDump: output to " << outfile << endl;
      ofstream of(outfile.c_str());
      printOutput(of);
    }
  } catch(SctConfiguration::ConfigurationException &e) {
    cout << "Exception writing configuration\n";
    cout << e.what() << endl;
  }
}

