#!/usr/bin/perl -w
use Fcntl;
use strict;
use CGI qw(param);

use lib "/userdisk/daquser/perllibs2";
use webconfig;


my ($outputfile,$resultHeader);
my @chipnames = qw {M0 S1 S2 S3 S4 E5 M8 S9 S10 S11 S12 E13};
my $noDataFlag=".";

my @tests = ("StrobeDelay","NPtGain","NoiseOccupancy");
my @referencefiles = ("HybDelay_Reference_warm.txt","Hyb3PGain_Reference_warm.txt","HybNoise_Reference_warm.txt");

################################################################# Do not edit anything after this line

my %testmap=();
my %datamap=();
my %corrmap=();
my %corr_runno=();
my %refdata=();

my %daqInfo=();
my %excludedruns=();

&getConfiguration();

foreach my $dataset (keys %config) {

  %excludedruns=();

  if($config{$dataset}{ACTIVE}==0) {
     print "Dataset $config{$dataset}{TITLE} is not active. Skipping...\n";
     next;
     }  
     
  print "Dataset $config{$dataset}{TITLE} is ACTIVE\n";

  my @eruns=split(/,/,$config{$dataset}{RUNEX});
  foreach (@eruns) {$excludedruns{$_}=1;}

  &readDaqTable($dataset);

  for(my $i=0;$i<3;$i++) {

     $outputfile = $tests[$i] . "_wwwsummary_${dataset}.txt";
     $resultHeader = $tests[$i] . "TestResult";
     %testmap=();
     %datamap=();
     %corrmap=();
     %corr_runno=();
     %refdata=();
     &collectLinks($dataset,$i);
     &getReferenceData($i);
     &printList($dataset,$i);
   }
}


### finished

sub bailout {

    my $error = "@_";

    print <<bailout_marker;
    <b> &gt Request Failed!</b>
    <br><br>
      Reason: $error
    <br><br>
    Sorry!
<br><br>
</body></html>
bailout_marker

die "$error";

}

sub readDaqTable {

      my ($this_index) = @_;
      %daqInfo=();

      my $lookup = "${lookupDir}$config{$this_index}{MOUNTS}";
      
      print "Reading daq table from $lookup\n";

      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open mount lookup file $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");

      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
#      foreach my $newmodule (keys %addedModules{$this_index}) {push(@lines,"$newmodule=$addedModules{$this_index}{$newmodule}");
      my $sctType = $config{$this_index}{TYPE};
      if(exists $addedModules{$sctType}) {
         for my $module (keys %{ $addedModules{$sctType} } ) {
	     print "Adding extra module $module=$addedModules{$sctType}{$module}\n";
	     push(@lines,"$module=$addedModules{$sctType}{$module}");
	     }
         }

 #     push(@lines,"20220380200139=Barrel 3, Row 27, Position 6");

      foreach my $line (@lines) {
        if($line=~/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            }
          }
      foreach my $rmodule (keys %replacedModules) {
          print "checking if $rmodule is possible\n";
	  my $type=$config{$this_index}{TYPE};
          if((exists $replacedModules{$rmodule}{TYPE}) && $replacedModules{$rmodule}{TYPE}==$type) {
	     print "It IS...\n";
	     $daqInfo{$replacedModules{$rmodule}{LAST}} = $daqInfo{$rmodule};
	     print "$rmodule now in same location as $replacedModules{$rmodule}{LAST} from Run $replacedModules{$rmodule}{FROMRUN}\n";
	     }
          }
	  
      my $count = (keys %daqInfo);
      print "# of modules in $config{$this_index}{TITLE} is $count\n";

#      close(IN) || printError("Cant close $lookup: $!");
      }




sub collectLinks {

my ($dataset,$index) = @_;

my $summaryDir = "$rootDirectory$config{$dataset}{DATADIR}/summaries/";

print "Looking in $summaryDir for summary files...\n";

opendir(HDATA,$summaryDir) || bailout("Cant open directory $summaryDir: $!");

while(my $filename = readdir(HDATA)) {

   if($filename=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {
   

      my $runno=$1;
      my $scanno = $2;
      my $sn = $3;

      if(!(exists $daqInfo{$sn})) {print "$sn is not a mounted module. skipping...\n"; next;}

      if($runno<$config{$dataset}{RUNLO}) {next;}
      if($runno>$config{$dataset}{RUNHI}) {next;}
      if(exists $excludedruns{$runno}) {print "Run $runno is excluded.skipping...\n"; next;}


      # Require 3PtGain only
      if($index==1) {
        my $nscans = getScanCount("$summaryDir$filename");
	if($nscans>3) {print "run $runno Scan $scanno does not have 3 scan points. skipping...\n"; next;}
	}

      if(!exists $testmap{$sn}) {
 	 $testmap{$sn}=$filename;
	 next;
	 }

      my $runscankey = $runno * 1e6 + $scanno;

      my $oldfile = $testmap{$sn};
      if($oldfile=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {
         my $thisrun = $1;
	 my $thisscan=$2;
	 my $thissn=$3;
	 my $thiskey = $thisrun * 1e6 + $thisscan;
         if($runscankey > $thiskey) {
	     $testmap{$sn}=$filename;
	     }
	 }
      }
  }

closedir(HDATA);
}

sub getReferenceData() {
    my ($index) = @_;
    my $filename = $lookupDir . $referencefiles[$index];

    printf "filename=$filename\n";
    
    if(! -e $filename) {printf "**Reference file $filename does not exist\n"; return;}
    
    open(IN,$filename) || bailout("cant open reference file $filename $!");
    flock(IN,1) || bailout("Cant flock $filename $!");

    while(<IN>) {
      if(/SN=(\d{14})TESTNO=(\d+)DATA=(.*)/) {
         if(exists $testmap{$1}) {$refdata{$1}=$3;}
	 }
      }
    close(IN) || bailout ("Cant close $filename $!");
    }


sub getCalCorrFactors {

 my ($dataset,$sn,$runno) = @_;

#     print "getting cal corrs for run $runno for module $sn\n";
#     print "===================================================\n";

     my $configfile = "$rootDirectory$config{$dataset}{DATADIR}/saved_data/configArchive_${runno}_start.xml.gz";

     if(!(-e $configfile)) {
        print "configfile $configfile does not exist\n";
	return "CFACTORS=CONF=";
	}

#     my $configfile = $dataDirectory . "configArchive_${runno}_start.xml";
     my $xmlcontent = `gunzip -c $configfile`;

     my @xml = split(/[\n\r]/,$xmlcontent);
     my $chipno=0;
     my $corrline="";
     my $foundit=0;
     my $confline="CONF=";
     my $id="xx";
     my $readdata=0;

     foreach my $line (@xml) {

       if($line=~/<module\s+id="(\d+)".*$sn.*/) {$id=$1; next;}
       if($line=~/<channel\s+id="${id}".*/) {$readdata=1;}
       if($readdata && $line=~/<\/channel>/) {$readdata=0; $id="xx"; next;}
       if($readdata) {
          if($line=~/<stream0\s+threshold="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
	      my $rx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.=";RX0:$rx" . ";RXD0:$dl";
#	      print "confline=$confline\n";
	      }
         if($line=~/<stream1\s+threshold="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
              my $rx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.=";RX1:$rx" . ";RXD1:$dl";
#	      print "confline=$confline\n";
	      }
         if($line=~/<output\s+current="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
              my $tx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.="TX:$tx" . ";TXD:$dl";
#	      print "confline=$confline\n";
	      }
         }


       if($line=~/<sn>$sn<\/sn>.*/) {$foundit=1;}
       if($foundit==0) {next;}

       if($line=~/<c_factor>(.*)<\/c_factor>/) {
	  my $corrfactor=$1;
	  if($corrfactor=~/(^Z.*)/) {printf "**ERROR $sn $chipnames[$chipno] Cal factor $1\n";}
	  else {
	     if($corrline ne "") {$corrline .=";";}
	     $corrline .= "$chipnames[$chipno]:$corrfactor"; $chipno++;
	     }
	  }
       if($line=~/<\/module>.*/) {last;}
       }

      if($chipno!=12) {
             print "**ERROR - Run $runno module $sn chipno=$chipno\n";
	     return "CFACTORS=CONF=";
	     }

#	print "RUN $runno $sn corrline=$corrline\n";
	     return "CFACTORS=$corrline$confline";



}





sub printList {

my ($dataset,$index) = @_;

my $modulecount=0;
my $outfile = "${lookupDir}$outputfile";

if(!(-e $outfile)) {
  open(OUTF,">$outfile") || bailout("Couldnt open $outfile: $!");
  close(OUTF) || bailout("cant close $outfile after initial open: $!");
  }

sysopen(OUTF,$outfile, O_RDWR) || bailout("Cant sysopen $outfile : $!");
flock(OUTF,2) || bailout("Cant flock $outfile $!");
seek(OUTF,0,0) || bailout("Cant seek $outfile: $!");
truncate(OUTF,0) || bailout("Cant truncate $outfile $!");

print "Opened $outfile for writing...\n";

my $deadcount=0;
my $totalcount=0;
my %defectHash=();

foreach my $sn (sort keys %testmap) {
   my $fname = $testmap{$sn};
   my $runno=0;
   my $scanno=0;
   if($fname=~/${resultHeader}\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {$runno=$1;$scanno=$2;}

   my $corrfactors = &getCalCorrFactors($dataset,$sn,$runno);
#   print "$sn = $testmap{$sn}\n";
   my $file = $rootDirectory . $config{$dataset}{DATADIR} . "/summaries/" . $testmap{$sn};
   my $loop=0;
   my $defectCount=0;
   my $firstdefect=-1;
   my $T0=$noDataFlag;
   my $T1=$noDataFlag;
   my $VDET=$noDataFlag;
   my $IDET=$noDataFlag;
   my $VCC=$noDataFlag;
   my $ICC=$noDataFlag;
   my $VDD=$noDataFlag;
   my $IDD=$noDataFlag;
   my $nextLineIsTemperatures=0;
   my $nextLineIsHV=0;
   my $nextLineIsVCC=0;
   my $nextLineIsVDD=0;
   my $chipoffset=-1;
   my $defectname="";
   my $firstchan=0;
   my $lastchan=0;
   my $line = "";
   open(IN,$file) || bailout("Cant open  file $file : $!");
    flock(IN,1) || bailout("Cant get LOCK_SH on $file: $!");
    my $nextline="";
    while(<IN>) {


      if(/^#T0\s*T1.*/) {$nextLineIsTemperatures=1; next;}
      if(/#VDET\s*IDET.*/) {$nextLineIsHV=1; next;}
      if(/#VCC\s*ICC.*/) {$nextLineIsVCC=1; next;}
      if(/#VDD\s*IDD.*/) {$nextLineIsVDD=1; next;}
      if($nextLineIsTemperatures==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $T0= ($1 eq ".") ? "." : sprintf("%.2f",$1);
	  $T1= ($2 eq ".") ? "." : sprintf("%.2f",$2);
	  }
	$nextLineIsTemperatures=0;
	}
      if($nextLineIsHV==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VDET=$1;
	  $IDET=$2;
	  }
	$nextLineIsHV=0;
	}
      if($nextLineIsVDD==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VDD=$1;
	  $IDD=$2;
	  }
	$nextLineIsVDD=0;
	}
      if($nextLineIsVCC==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VCC=$1;
	  $ICC=$2;
	  }
	$nextLineIsVCC=0;
	}

    if($index==1) {                      #nptgain
      if(/^#LoopB.*/) {$loop++;}
      if($loop==0) {next;}
      if(/^#([MSE]\d{1,2})$/) {$nextline=$1; next;}
      if($nextline ne "") {
        my @params = split(/\s+/);
	my $nseoffset = (@params==9) ? 7 : 8;
	my $gainoffset = (@params==9) ? 2 : 3;
	my $nse = $params[$nseoffset];
	my $gain = $params[$gainoffset];

	if($line ne "") {$line .= ";";}
        $line .= "${nextline}:${nse}*$gain";
#	print "$sn $nextline = $nse\n";
	}
      $nextline="";
      }
    elsif($index==2) {                      #noise

      if(/^#([MSE]\d{1,2})$/) {$nextline=$1; next;}
      if($nextline ne "") {
        my @params = split(/\s+/);
	my $occoffset = (@params == 4) ? 1 : 2;
	my $nseoffset = (@params == 4) ? 3 : 4;
	my $occ = $params[$occoffset];
	my $nse = $params[$nseoffset];
	if($line ne "") {$line .= ";";}
        $line .= "${nextline}:${nse}*$occ";
#	print "$sn $nextline = $nse\n";
	}
      $nextline="";
      }
    elsif($index==0) {                    # strobedelay
      if(/^#M0\s+S1\s+S2\s+S3\s+S4\s+E5.*/) {$chipoffset=0; next;}
      if(/^#M8\s+S9\s+S10\s+S11\s+S12\s+E13.*/) {$chipoffset=6; next;}
      if($chipoffset != -1) {
         my @chipdata = split(/\s+/);
	 my $miny=$chipoffset;
	 my $maxy=6+$chipoffset;
	 for(my $y=$miny;$y<$maxy;$y++) {
	    if($line ne "") {$line .= ";";}
	    $line .=$chipnames[$y] . ":$chipdata[$y-$chipoffset]";
	    }

         }
      $chipoffset=-1;
      }

      if(/#(\d+)\s+defects\s+found\s+affecting\s+(\d+)\s+strips.*/) {$defectCount=$2;}
      
      if(/DEFECT\s*NAME\s*:\s*(.*)/) {$defectname=$1; next;}
      if(/FIRST\s*CHANNEL\s*:\s*(\d+)/) {$firstchan=$1; next;}
      if(/LAST\s*CHANNEL\s*:\s*(\d+)/) {
          if(!(exists $defectHash{$defectname})) {$defectHash{$defectname}=0;}
          my $lastchan=$1;
	  for(my $y=$firstchan;$y<=$lastchan;$y++) {
	     $defectHash{$defectname}++;
	     $totalcount++;
	     }
	  $defectname="";
	  }





      }
  printf "$tests[$index] Run $runno Scan $scanno $sn\n";

  my $DCS="${T0}*${T1}*${VDET}*${IDET}*${VCC}*${ICC}*${VDD}*${IDD}";
  
  my $REFDATA = (exists $refdata{$sn}) ? "REF=".$refdata{$sn} : "REF=";

  print OUTF "RUN=${runno}SCAN=${scanno}SN=${sn}DATA=${line}DCS=${DCS}DEFECTS=${defectCount}${corrfactors}${REFDATA}\n";
  $modulecount++;


  close(IN) || bailout("Cant close file $file : $!");
  


   }
   foreach my $defect (sort keys %defectHash) {
     print OUTF "DEFECT${defect}=$defectHash{$defect}\n";
     }

   print OUTF "TOTALDEFECTCOUNT=${totalcount}\n";
    printf "Saved $tests[$index] data for $modulecount modules\n";
    close(OUTF) || bailout("Cant close $outfile : $!");
}

sub getScanCount() {
 my $count=0;
 my $fullname = shift;

 open(IN,$fullname) || bailout("Cant open $fullname for scan count");
 flock(IN,1) || bailout("Cant flock $fullname");
 
 my $nextLineIsCount=0;

 while(<IN>) {
   if(/^[\s]?#N_POINTS/) {$nextLineIsCount=1; next;}
   if($nextLineIsCount) {
     if(/[\s]?(\d+)/) {
       $count=$1;
       last;
       }
     }
   }
 close(IN) || bailout("Cant close $fullname");
 return $count;
 }
 
 sub getIntFromHex {
   my $hex = shift;
   if($hex=~/(0x[a-fA-F\d]+)/) {
     return hex $hex;
     }
   return $hex;
   }








