package sctroddaqtests;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(parseSummaryData dieError printModuleTestList addZeros removeZeros getNumberOfModules);
@EXPORT_OK = qw($summaryDir @chipNames);

$VERSION = 1.00;

use strict;
use Fcntl;


my @chipNames = ("Module Average","Chip Maximum","Chip Minimum","Mean Link0","Mean Link1","Chip M0","Chip S1","Chip S2","Chip S3","Chip S4","Chip E5","Chip M8","Chip S9","Chip S10","Chip S11","Chip S12","Chip E13");



sub parseSummaryData {

my($summaryDir,$runno,$scanno,$testNameHeader,$sn,$backlinks) = @_;


 $runno = sprintf("%d",$runno);
 if($runno eq "" || $testNameHeader eq "" || $sn eq "") {dieError("invalid arguments to script");}

 my $fname = "${summaryDir}${testNameHeader}.${runno}.${scanno}.${sn}.summary";

 if(!-e $fname) {dieError("$fname does not exist.");}

 open(IN,$fname) || dieError("Cant open $fname $!");
 flock(IN,1) || dieError("Cant flock $fname $!");



 my $printedDAQInfo=0;
 my $printedDCSInfo=0;
 my $printedDefectTable=0;
 my $delayData=0;
 my $printedTableData=0;

 my $nextLineIsHost=0;
 my $nextLineIsTemperatures=0;
my $nextLineIsHV=0;
my $nextLineIsVCC=0;
my $nextLineIsVDD=0;
my $nextLineIsTime=0;

my $nextLineIsScanCount=0;
my $nextLineIsScanPoints=0;
my $nextLineIsDelays=0;

my $printLines=0;


 my $passflag="Unknown";
 my $problemflag="Unknown";
 my $date="Unknown";
 my $host="Unknown";
 my $time="Unknown";

 my $chan1="Unknown";
 my $defectname = "Unknown";

 my $scanCount=0;
 my @scanPoints=();

 my $table=0;
 my $nextLineIsTableData=0;

 print qq!
 <h3>$testNameHeader Test Report for ${sn}, Run $runno, Scan $scanno </h3>
 $backlinks
 <br>
 !;



 while(<IN>) {
   if(/^\s+(.*)/) {$_=$1;}
# printf "$_ <br>\n";
   if(/.*PASSED\s*:\s*(.*)/) {$passflag=$1;}
   if(/.*PROBLEM\s*:\s*(.*)/) {$problemflag=$1;}
   if(/.*TEST[ _]+DATE\s*:\s*(.*)/) {$date=$1;}
   if(/^#HOST.*/) {$nextLineIsHost=1; next;}
   if(/^#\s*TIME.*/) {$nextLineIsTime=1; next;}

   if($nextLineIsHost==1) {
      if(/.*"(.*)"/) {$host=$1;}
      $nextLineIsHost=0;
      }
   if($nextLineIsTime==1) {
      if(/.*"(.*)"[;]?/) {$time=$1;}
      $nextLineIsTime=0;
      }


   if(/[\%0D]+CS_INFO.*/) {
     $printedDAQInfo=1;
     printDAQInfo($host,$date,$time,$passflag,$problemflag);
     print qq!


     <b>DCS Info</b>
     <table border=1>
     <tr><td><b>T0</b></td><td><b>T1</b></td><td><b>TDiff</b></td><td><b>VDet</b></td><td><b>IDet</b></td><td><b>Vcc</b></td><td><b>Icc</b></td><td><b>Vdd</b></td><td><b>Idd</b></td></tr>
     !;
     }

   if(/#T0\s*T1.*/) {$nextLineIsTemperatures=1; next;}
   if(/#VDET\s*IDET.*/) {$nextLineIsHV=1; next;}
   if(/#VCC\s*ICC.*/) {$nextLineIsVCC=1; next;}
   if(/#VDD\s*IDD.*/) {$nextLineIsVDD=1; next;}

   if($nextLineIsTemperatures==1) {
     my ($t0,$t1) = split(/\s+/);

     printf "<tr><td>". sprintf("%.1f",$t0) ."</td><td>" . sprintf("%.1f",$t1) . "</td>";
     my $tdiff="Unknown";
     if($t0 ne "." && $t1 ne ".") {$tdiff = sprintf("%.1f",abs($t0-$t1));}
     printf "<td>$tdiff</td>";
     $nextLineIsTemperatures=0;
     }
    if($nextLineIsHV==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td>";
     $nextLineIsHV=0;
     }
    if($nextLineIsVCC==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td>";
     $nextLineIsVCC=0;
     }
    if($nextLineIsVDD==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td></tr></table><br>";
     $printedDCSInfo=1;
     $nextLineIsVDD=0;
     }

   if(/^#N_POINTS.*/) {$nextLineIsScanCount=1; next;}
   if(/^#POINTS.*/) {$nextLineIsScanPoints=1; next;}

   if($nextLineIsScanCount) {
     $scanCount=$_;
     $nextLineIsScanCount=0;
     }
   if($nextLineIsScanPoints) {
     if(/^#.*/) {printScanData($scanCount,@scanPoints); $nextLineIsScanPoints=0;}
     my @p = split(/\s+/);
     foreach my $point (@p) {
        if($point ne ".") {push(@scanPoints,$point);}
	}
     }

   if(/^#LoopA.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>RC Fits</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Func</b></td><td><b>P0</b></td><td><b>P1</b></td><td><b>P2</b></td></tr>
      !;
      }
   elsif(/^#LoopB.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>RC Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>vt50</b></td><td><b>Vt50 RMS</b></td><td><b>Gain</b></td><td><b>Gain RMS</b></td><td><b>Offset</b></td><td><b>Offset RMS</b></td><td><b>Output Noise</b></td><td><b>ENC input Noise</b></td><td><b>ENC noise RMS</b></td></tr>
      !;
      }
   elsif(/^#\s*Offset\s*MeanOcc\s*RMSOcc\s*EstENC.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Occupancy Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Offset</b></td><td><b>Occupancy</b></td><td><b>Occupancy RMS</b></td><td><b>ENC Noise</b></td></tr>
      !;
      }
   elsif(/[#]?\s*range\s*target\s*ntrim\s*vt50\s*vt50rms\s*tr_off\s*off_rms\s*tr_step.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Trim Data</b>
      <table border=1>
      <tr><td><b>Range</b></td><td><b>Target</b></td><td><b>NTrim</b></td><td><b>vt50</b></td><td><b>vt50(rms)</b></td><td><b>Trim Offset</b></td><td><b>Trim Offset (rms)</b></td><td><b>Trim Step</b></td></tr>
      !;
      }
   elsif(/^%NMaskTest.*/) {
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      $printedTableData++;
      print qq!
      <b>Test Data</b><br>
      There is no test data for this type of test.
      <br><br>
      !;
       }

   if(/^#([MSE]+\d{1,2})$/) {
      $nextLineIsTableData=1;
      printf "<tr><td nowrap>$1</td>";
      next;
      }
   if($table && (/^#$/ || /^%Defect.*/)) {
      $table=0;
      $printedTableData++;
      printf "</table><br>";
      }
   if($nextLineIsTableData) {
     my @dataline = split(/\s+/);
     foreach my $databit (@dataline) {
       printf "<td nowrap>${databit}&nbsp;&nbsp;&nbsp;</td>";
       }
     printf "</tr>";
     $nextLineIsTableData=0;
     }
     
   if(/^%FullBypassTest.*/ || /^%TimeWalkTest.*/) {
     if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
     if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
     $printLines=1;
     }

   if(/#DELAY/) {
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Delay Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Delay</b></td></tr>
      !;
      $delayData=1;
      next;
      }
   if(/^#M0\s+S1\s+S2\s+S3\s+S4\s+E5/ && $delayData) {$nextLineIsDelays=1; next;}
   if(/^#M8\s+S9\s+S10\s+S11\s+S12\s+E13/ && $delayData) {$nextLineIsDelays=7; next;}
   if($nextLineIsDelays) {
     my @delays = split(/\s+/);
     my $ycount=4+$nextLineIsDelays;
     foreach my $delay (@delays) {
        printf "<tr><td>$chipNames[$ycount]&nbsp;&nbsp;</td><td>$delay&nbsp;&nbsp;</td></tr>\n";
	$ycount++;
	}
     if($nextLineIsDelays==7) {printf "</table><br>"; $printedTableData++;}
     $nextLineIsDelays=0;
     }

    if(/^%Defect.*/) {
      $printLines=0;
      }

   if(/DEFECT\s+NAME\s*:\s*(.*)/) {
      if($printedDefectTable==0) {printDefectTable(); $printedDefectTable=1;}
      $defectname=$1;
      }
   if(/FIRST\s+CHANNEL\s*:\s*(\d+)/) {$chan1=$1;}
   if(/LAST\s+CHANNEL\s*:\s*(\d+)/) {
     my $chans = ($chan1 eq $1) ? $chan1 : $chan1 . " - " . $1;
     printf "<tr><td>$chans &nbsp;&nbsp;</td><td>$defectname</td></tr>\n";
     }
   if(/#(\d+\s*defects\s*found\s*affecting\s*(\d+)\s*strips)/) {
      printf "</table><br>";
      printf "&gt;&nbsp;&nbsp;$1 <br>\n";
      }

   if(/#No\s*defects\s*found/) {
     $printLines=0;
     print qq!

       <b>Defects</b><br>
       None found.
       <br><br>
       !;
       }
       
   if($printLines) {print "$_<br>\n";}






   }
 close(IN) || dieError("Cant close $fname $!");

 if($printedTableData==0) {
       if($printedDAQInfo==0) {printDAQInfo($host,$date,$time,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Test Data</b><br>
      Sorry, test data reports are not available yet for this type of test.
      !;
      }


 }

  sub printDefectTable() {
   print qq!
   <b>Defects</b>
   <table border=1>
   <tr><td><b>Channel(s)</b></td><td><b>Defect</b></td></tr>
   !;
   }
 sub printDAQInfo() {
   my ($host,$date,$time,$passflag,$problemflag)=@_;
   print qq!
    <b>DAQ info</b>
     <table border=1>
     <tr><td><b>Host</b></td><td><b>Date</b></td><td><b>Time</b></td><td><b>Passed?</b></td><td><b>Problem?</b></td></tr>
     <tr><td>$host</td><td>$date</td><td>$time</td><td>$passflag</td><td>$problemflag</td></tr>
     </table>
     <br>
     !;
     }
 sub printScanData() {
   my ($pointcount,@points)=@_;
   print qq!
    <b>Scan Info</b>
     <table border=1>
     <tr><td><b>#Scans</b></td><td><b>Points</b></td></tr>
     <tr><td>$pointcount</td><td>
     !;
     my $count=0;
     foreach my $p (@points) {
       if($p ne ".") {
          if($count>0) {printf ", ";}
	  $count++;
	  printf "$p";
	  }
	}
    print qq!
     </td></tr>
     </table>
     <br>
     !;
     }
 sub printDCSInfo() {
    print qq!
      <b>DCS Info</b><br>
      Not available.
      <br><br>
      !;
      }

sub printModuleTestList {

 my ($summaryDir,$dataSet, $sn,$scriptname,$linkback) = @_;

 my %fileHash=();
 my %infoHash=();

 
 opendir(HDATA,$summaryDir) || dieError("Cant open directory $summaryDir: $!");

 while(my $fname = readdir(HDATA)) {
   if($fname=~/(.*)TestResult.(\d+).(\d+).${sn}.summary/) {
      my $tname=$1;
      my $runno=$2;
      my $scanno=$3;
      my $fullfile = "$summaryDir$fname";
      my $testdate="Unknown";
      my $time="Unknown";
      my $passflag="Unknown";
      my $nextLineIsTime=0;

      open(IN,$fullfile) || printError("Cant open $fullfile");
      flock(IN,1) || printError("Cant flock $fullfile");
      while(<IN>) {
        if(/^TEST[_\s]DATE\s*:\s*(.*)/) {$testdate=$1; next;}
	if(/^#TIME.*/) {$nextLineIsTime=1; next;}
	if($nextLineIsTime) {
	   if(/"(.*)"/) {$time=$1;}
	   last;
	   }
	if(/PASSED\s*:\s*(.*)/) {$passflag=$1;}
	}
      close(IN) || printError("Cant close $fullfile");

      my $link = "<a href=$scriptname\?getTest=1\&sn=$sn\&test=$tname\&runno=$runno\&scanno=$scanno\&dataSet=$dataSet>$tname</a>";
      my $key = $scanno + ($runno * 1000000);
      $fileHash{$key}=$link;
      $infoHash{$key}="${passflag};${testdate};$time";
      }
   }
 closedir (HDATA);

 print qq!
 <h3>Test History of $sn </h3>
 $linkback

 <table border=0 cellspacing=0 cellpadding=0>
 <col width=140>
 <col width=60>
 <col width=60>
 <col width=80>
 <col width=80>
 <col width=60>
 <tr><td><b>Test</b></td><td><b>Run</b></td><td><b>Scan</b></td><td><b>Date</b></td><td><b>Time</b></td><td><b>Status</b></td></tr>
 !;
 my $lineCount=0;
 foreach my $key (sort keys %fileHash) {
  my $runno = $key/1000000;
  $runno =sprintf("%d",$runno);
  my $scanno = $key%1000000;
  my $background = ($lineCount%2==0) ? "bgcolor=\"#f6f6f6\"" : "bgcolor=\"#eeeeff\"";
   my ($passflag,$date,$time) = split(/;/,$infoHash{$key});
   my $pass = ($passflag eq "YES") ? "Passed" : "<font color=red>FAIL</font>";
   my $run = "<a href=http://www-pnp.physics.ox.ac.uk/~daquser/cgi-bin/sctroddaqsummary.cgi\?listtest=1\&runno=$runno\&logbookIndex=$dataSet>$runno</a>";
   print "<tr $background><td>$fileHash{$key}</td><td>$run</td><td>$scanno</td><td>$date</td><td>$time</td><td>$pass</td></tr>\n";
   $lineCount++;
   }
  print "</table><br>";

}

sub addZeros {
  my ($scan) = @_;
  $_=$scan;
  my $length = tr/0-9//;
  for(my $i=$length;$i<6;$i++) {
     $scan = "0".$scan;
     }
  return $scan;
  }
  
sub removeZeros {
  my ($number)=@_;
  if($number=~/0+([1-9]+[0-9]*)/) {return $1;}
  return $number;
  }
  
sub getNumberOfModules {
 my $runno = shift;

 $runno = addZeros($runno);

 my $indexFile = "/userdisk/daquser/WebLookupData/bookkeeping/$runno/index.txt";
 if(!(-e $indexFile)) {return 0;}   # require at least one test
 open(IN,$indexFile) || printError("Cant open $indexFile: $!");
 flock(IN,1) || printError("Cant get LOCK_SH on $indexFile: $!");

 my $nomodules=0;
  while(<IN>) {
#	  RUN=1367;NAME=NMaskTest;START=1;NSCANS=1;DATE=17/08/2004;STIME=20040817T100755;ETIME=20040817T100909;STATUS=1;NMODULES=14
     if(/RUN=${runno}.*NMODULES=(\d+)/) {
       $nomodules=$1;
       printf "***found noModules=$nomodules <br>\n";
       last;
       }
  }
 close(IN) || printError("Casnt close $indexFile : $!");
 return $nomodules;
 }







sub dieError {

    my $error = "@_";

    print <<dieError_marker;
    <b> Error!</b>
    <br><br>
      Reason: $error
    <br><br>
    Sorry! Please <a href="mailto:robinson\@hep.phy.cam.ac.uk">let me know</a> and I'll try to fix it.
<br>
dieError_marker

die $error;

}



1;
