#!/usr/bin/perl -w

#
# Barrel Data display script
# Dave Robinson
#

use Fcntl;
use strict;
use CGI qw(param);
use lib "/userdisk/daquser/perllibs2";
use sctroddaqtests;
use webconfig;

use CGI::Carp qw(fatalsToBrowser);

my @chipNames = ("Module Average","Chip Maximum","Chip Minimum","Mean Link0","Mean Link1","Chip M0","Chip S1","Chip S2","Chip S3","Chip S4","Chip E5","Chip M8","Chip S9","Chip S10","Chip S11","Chip S12","Chip E13");

my @nptgain_param_names = ("ENC Noise","Gain (mV/fC)","ENCnoise (SctRodDaq-SCTDAQ)","Gain (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff","BOC Tx","BOC TXDelay","BOC RX Thr stream0","BOC RX Dly Stream0","BOC RX Thr Stream1","BOC RX Dly Stream1");
my @occ_param_names = ("ENC Noise","Occupancy","ENCnoise (SctRodDaq-SCTDAQ)","Occupancy (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff","BOC Tx","BOC TXDelay","BOC RX Thr stream0","BOC RX Dly Stream0","BOC RX Thr Stream1","BOC RX Dly Stream1");
my @delay_param_names = ("Delay","Delay (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff","BOC Tx","BOC TXDelay","BOC RX Thr stream0","BOC RX Dly Stream0","BOC RX Thr Stream1","BOC RX Dly Stream1");
my @cellfills = ("Data Value","Serial No","cFactor","None");

my $scriptname = $ENV{'SCRIPT_NAME'};  # name of this script for links etc

my $noDataString = "No Data";

my @barrelCount = (0,0,0,0);
my @barrelRows = (32,40,48,56);
my @testIndexNames = qw{NPtGain NoiseOccupancy StrobeDelay};
my @parameterIndexNames = qw{enc occ};

my %colorHashC0 = (
   "ENC Noise" => [1200, 0, 0, 255 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [30,0,0,255],
   "Defective Channels" => [0,0,255,0],
   "DCS Data - T0" => [0,0,0,255],
   "DCS Data - T1" => [0,0,0,255],
   "DCS Data - VDet" => [0,0,0,255],
   "DCS Data - IDet(nA)" => [0,0,0,255],
   "Hybrid TDiff" => [0,0,255,0],
   "DCS Data - Vdd" => [3.5,0,0,255],
   "DCS Data - Idd" => [450,0,0,255],
   "DCS Data - Vcc" => [3.0,0,0,255],
   "DCS Data - Icc" => [800,0,0,255],
   "Delay" => [0,0,0,255],
   "Occupancy" => [0.0000001,0,0,255],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [-150,0,0,255],
   "Gain (SctRodDaq-SCTDAQ)" => [-10,0,0,255],
   "Occupancy (SctRodDaq-SCTDAQ)" => [-0.00005,0,0,255],
   "Delay (SctRodDaq-SCTDAQ)" => [-5,0,0,255],
   "BOC Tx" => [0,0,0,255],
   "BOC TXDelay" => [0,0,255,0],
   "BOC RX Thr stream0" => [0,0,0,255],
   "BOC RX Dly Stream0" => [0,0,255,0],
   "BOC RX Thr Stream1" => [0,0,0,255],
   "BOC RX Dly Stream1" => [0,0,255,0]
   );
   
my %colorHashC1 = (
   "ENC Noise" => [1400, 0, 255, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [40,0,255,0],
   "Defective Channels" => [5,255,255,0],
   "DCS Data - T0" => [20,0,255,0],
   "DCS Data - T1" => [20,0,255,0],
   "Hybrid TDiff" => [1,255,255,0],
   "DCS Data - VDet" => [150,0,255,0],
   "DCS Data - IDet(nA)" => [400,0,255,0],
   "DCS Data - Vdd" => [3.8,0,255,0],
   "DCS Data - Idd" => [500,0,255,0],
   "DCS Data - Vcc" => [3.4,0,255,0],
   "DCS Data - Icc" => [950,0,255,0],
   "Delay" => [10,0,255,0],
   "Occupancy" => [0.000001,0,255,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [-75,0,255,0],
   "Gain (SctRodDaq-SCTDAQ)" => [-5,0,255,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [-0.00002,0,255,0],
   "Delay (SctRodDaq-SCTDAQ)" => [-2,0,255,0],
   "BOC Tx" => [100,0,255,0],
   "BOC TXDelay" => [10,0,255,0],
   "BOC RX Thr stream0" => [100,0,255,0],
   "BOC RX Dly Stream0" => [10,0,255,0],
   "BOC RX Thr Stream1" => [100,0,255,0],
   "BOC RX Dly Stream1" => [10,0,255,0]
   );
my %colorHashC2 = (
   "ENC Noise" => [1600, 0, 255, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [60,0,255,0],
   "Defective Channels" => [10,255,255,0],
   "DCS Data - T0" => [30,0,255,0],
   "DCS Data - T1" => [30,0,255,0],
   "Hybrid TDiff" => [3,255,255,0],
   "DCS Data - VDet" => [350,0,255,0],
   "DCS Data - IDet(nA)" => [600,0,255,0],
   "DCS Data - Vdd" => [4.2,0,255,0],
   "DCS Data - Idd" => [550,0,255,0],
   "DCS Data - Vcc" => [3.6,0,255,0],
   "DCS Data - Icc" => [1050,0,255,0],
   "Delay" => [20,0,255,0],
   "Occupancy" => [0.00001,0,255,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [75,0,255,0],
   "Gain (SctRodDaq-SCTDAQ)" => [5,0,255,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [0.00002,0,255,0],
   "Delay (SctRodDaq-SCTDAQ)" => [2,0,255,0],
   "BOC Tx" => [160,0,255,0],
   "BOC TXDelay" => [16,0,255,0],
   "BOC RX Thr stream0" => [140,0,255,0],
   "BOC RX Dly Stream0" => [16,0,255,0],
   "BOC RX Thr Stream1" => [140,0,255,0],
   "BOC RX Dly Stream1" => [16,0,255,0]
   );

my %colorHashC3 = (
   "ENC Noise" => [1800, 255, 0, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [70,255,0,0],
   "Defective Channels" => [15,255,0,0],
   "DCS Data - T0" => [40,255,0,0],
   "DCS Data - T1" => [40,255,0,0],
   "Hybrid TDiff" => [4,255,0,0],
   "DCS Data - VDet" => [500,255,0,0],
   "DCS Data - IDet(nA)" => [1000,255,0,0],
   "DCS Data - Vdd" => [4.5,255,0,0],
   "DCS Data - Idd" => [600,255,0,0],
   "DCS Data - Vcc" => [4.0,255,0,0],
   "DCS Data - Icc" => [1200,255,0,0],
   "Delay" => [30,255,0,0],
   "Occupancy" => [0.0001,255,0,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [150,255,0,0],
   "Gain (SctRodDaq-SCTDAQ)" => [10,255,0,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [0.00005,255,0,0],
   "Delay (SctRodDaq-SCTDAQ)" => [5,255,0,0],
   "BOC Tx" => [200,255,0,0],
   "BOC TXDelay" => [20,255,0,0],
   "BOC RX Thr stream0" => [200,255,0,0],
   "BOC RX Dly Stream0" => [20,255,0,0],
   "BOC RX Thr Stream1" => [200,255,0,0],
   "BOC RX Dly Stream1" => [20,255,0,0]
   );




################################################################# Do not edit anything after this line

print "Content-type: text/html\n\n";

&getConfiguration();

my $logScript = "${cgi_base}sctroddaqsummary.cgi";

if(-e "${lookupDir}maintenancefile.txt") {
 print qq!
 <html>
 <head>
 <title>Barrel Display not available</title>
 </head>
 <body>
 <h3>The Barrel Display is not available</h3>
 Sorry, the barrel data display is not available, due to maintenance.
 <br><br>
 Please try again in a few moments.
 <br><br>
 If this message persists, please <a href="mailto:robinson\@hep.phy.cam.ac.uk>complain</a>.
 </body>
 </html>
 !;
 die;
 }

my %daqInfo = ();
my %daqMapping=();
my %dataMap=();
my %cMap=();
my %defectCount=();
my $totaldefectcount=-1;
my $daqInfoExists=0;

my %datalinks=();
my $originInfoExists=0;
my $moduleMountMapIsValid=0;

my $moduleCount=0;

# testIndex = index of test name
# parameterIndex = index of displayed parameter
# chip = index of chip option

my $testIndex = param("testIndex") ? param("testIndex") : 0;
my $parameterIndex = param("parameterIndex") ? param("parameterIndex") : 0;
my $chipIndex = param("chipIndex") ? param("chipIndex") : 0;
my $cellfillIndex = param("cellfillIndex") ? param("cellfillIndex") : 0;
my $dataSet = param("dataSet") ? param("dataSet") : 0;

my @paramlist = @nptgain_param_names;
if($testIndex==1) {@paramlist = @occ_param_names;}
elsif($testIndex==2) {@paramlist = @delay_param_names;}

my $isOccupancyData = ($testIndex==1 && ($parameterIndex==1 || $parameterIndex==3)) ? 1 : 0;

my $dataPointer = $parameterIndex;


SWITCH: {
    param("showrawdata") && do {&printPageHeader("Raw Data"); &printRawData($testIndex,$parameterIndex,$chipIndex); last SWITCH;};
    param("getTest") && do {&printPageHeader("Test Data"); &showTestData(); last SWITCH;};
    param("getList") && do{&printPageHeader("Test List");showTestList(); last SWITCH;};
    param("showDataSetInfo") && do{&printPageHeader("DataSet Information"); showDataSetInfo(); last SWITCH;};

    &printPageHeader("SctRodDaq Data Display");
    &readDaqTable($config{$dataSet}{TYPE});
    &readParamTable($testIndex,$parameterIndex,$chipIndex);
    &printBarrelFile($config{$dataSet}{TYPE});
    my $sname = "$scriptname\?testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex\&showrawdata=1";
    print "<br><br>&gt;&nbsp;<a href=\"#\" onClick=\"window.open('$sname','RawData','menubar=1,scrollbars=1'); return false;\">View the raw data</a> used to generate this web page.";

}


print qq!
</body>
</html>
!;

sub printRawData() {

    my ($testIndex,$parameterIndex,$chipIndex) = @_;

    my $fname = "${lookupDir}$testIndexNames[$testIndex]_wwwsummary_$dataSet.txt";


    open(IN,$fname) || printError("Cant open $fname: $!");
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
    
    print qq!
<html>
<head>
<title>Raw Data</title>
<link rel=stylesheet HREF="http://www.hep.phy.cam.ac.uk/~robinson/styles/davestyles2.css" type="text/css">
!;
print "<\!-- Raw data from file $fname -->";
print qq!
</head>
<body>
!;
   while(<IN>) {
      print "$_ <br>\n";
      }
    close(IN) || printError("Cant close $fname: $!");
    }

sub printError {
 my ($reason) = @_;

   print qq!
  <html>
<head>
<title>Barrel Module Positions</title>
</head>
<body>

<h3><font color=red>Error</font></h3>
<br>
$reason
<br><br>

</body>
</html>
!;
die;
}

sub readParamTable() {

    my ($testIndex,$parameterIndex,$chipIndex) = @_;

    my $fname = "$lookupDir$testIndexNames[$testIndex]_wwwsummary_$dataSet.txt";

    open(IN,$fname) || printError("Cant open $fname: $!");
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
#RUN=1371SCAN=0SN=20220040200007NOISE=M0:1183.21;S1:1211.68;S2:1293.97;S3:1283.25;S4:1257.9;E5:1254.65;M8:1220.48;S9:1280.05;S10:1266.41;S11:1214.65;S12:1338.63;E13:1307.26CFACTORS=M0:1.00;S1:1.00;S2:1.00;S3:1.00;S4:1.00;E5:1.00;M8:1.00;S9:1.00;S10:1.00;S11:1.00;S12:1.00;E13:1.00
    while(<IN>) {
       if(/^DEFECT(.*)=(\d+)/) {$defectCount{$1}=$2;}
       if(/^TOTALDEFECTCOUNT=(\d+)/) {$totaldefectcount=$1;}
       if(/.*RUN=(\d+)SCAN=(\d+)SN=(\d{14})DATA=(.*)DCS=(.*)DEFECTS=(.*)CFACTORS=(.*)CONF=(.*)REF=(.*)/) {
            my $runno=$1;
	    my $scanno=$2;
	    my $sn=$3;
	    my $dataline=$4;
	    my $dcsline=$5;
	    my $defectCount=$6;
	    my $cfactorline=$7;
	    my $confline=$8;
	    my $refline=$9;
	    
	    if(exists $replacedModules{$sn}) {
	       if($runno<$replacedModules{$sn}{FROMRUN}) {next;}
	       }

	    $datalinks{$sn}="&sn=$sn\&test=$testIndexNames[$testIndex]\&runno=$runno\&scanno=$scanno\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillIndex=$cellfillIndex";

	    my @chipdata = split(/;/,$dataline);
	    my @cfactordata = split(/;/,$cfactorline);
	    my @refdata = split(/;/,$refline);
	    my $chipCount=0;
	    my $chipCount0=0;
	    my $chipCount1=0;
	    my $thisData=0.;
	    my $thisRefData=0.;
	    my $cData=0.;
	    
	    my $defectIndex= ($testIndex==2) ? 2 : 4;
	    my $dcsIndex = ($testIndex==2) ? 3 : 5;
	    my $tdiffIndex = $dcsIndex+8;
	    my $confIndex=$tdiffIndex+1;
	    my $refIndex = ($testIndex==2) ? 1 : 2;

	    if($parameterIndex==$defectIndex) {
	       $thisData=$defectCount;
               $dataMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn}) {$moduleCount++;}
	       next;
	       }
	       
	    if($parameterIndex>=$confIndex) {
	       my @confbits = split(/;/,$confline);
	       my $cbit = $confbits[$parameterIndex-$confIndex];
	       if($cbit=~/[RXTD01]+:(\d+)/) {$thisData=$1; $dataMap{$sn}=$thisData;}
	       if(exists $daqInfo{$sn}) {$moduleCount++;}
	       next;
	       }
	       

	    if($parameterIndex>=$dcsIndex) {
	       my @thisline= split(/\*/,$dcsline);
	       if($parameterIndex<$tdiffIndex) {$thisData = ($thisline[$parameterIndex-$dcsIndex] eq ".") ? $noDataString : $thisline[$parameterIndex-$dcsIndex];}
	       else {
  	          if($thisline[1] ne "." && $thisline[0] ne ".") {$thisData = abs($thisline[0]-$thisline[1]); $thisData = sprintf("%.2f",$thisData);}
		 else {$thisData = $noDataString;}

		 }
               $dataMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn}) {$moduleCount++;}
	       next;
	       }

	    my $doComparison=0;

	    $dataPointer=$parameterIndex;
	    if($parameterIndex>=$refIndex) {
	       $doComparison=1;
	       $dataPointer = $parameterIndex-$refIndex;
	       }
	       
	    my $scaleReferenceData = !($testIndex==2 || ($testIndex==1 && $dataPointer==1));

            if($chipIndex<=4) {  # mean overall
	        my $total=0.;
		my $totalRef=0.;
		my $minValue=99999.;
		my $maxValueRef=0.;
		my $minValueRef=0.;
		my $maxValue=0.;
		my $mean0=0.;
		my $meanRef0=0.;
		my $mean1=0.;
		my $meanRef1=0.;
	        $thisData=0.;
		$thisRefData=0.;
		$cData=0.;
	        for(my $i=0;$i<12;$i++) {
		   my ($chipName,$dataStar) = split(/:/,$chipdata[$i]);
		   my @datas = split(/\*/,$dataStar);
		   my @reference_datas = split(/\*/,$refdata[$i]);
		   my $data = $datas[$dataPointer];
		   my $reference = ($doComparison) ? $reference_datas[$dataPointer] : 0.;
		   my ($cname,$cdata) = split(/:/,$cfactordata[$i]);
		   if($scaleReferenceData) {$reference *= $cdata;}
		   if($data eq "nan" || $data eq "0") {next;}
		   $total += $data;
		   $totalRef += $reference;
		   if($i<6) {$mean0 += $data; $meanRef0 += $reference; $chipCount0++;}
		   if($i>=6) {$mean1 += $data; $meanRef1 += $reference; $chipCount1++;}
		   if($data<$minValue) {$minValue=$data; $minValueRef=$reference;}
		   if($data>$maxValue) {$maxValue=$data; $maxValueRef=$reference;}
		   $cData += $cdata;
		   $chipCount++;
		   }

		if($chipIndex==0) {
		   $thisData = ($chipCount>0) ? $total/$chipCount : $noDataString;
		   $thisRefData = ($chipCount>0) ? $totalRef/$chipCount : $noDataString;
		   }
		elsif($chipIndex==1) {$thisData = $maxValue; $thisRefData=$maxValueRef;}
		elsif($chipIndex==2) {$thisData = $minValue; $thisRefData=$minValueRef;}
		elsif($chipIndex==3) {
		    $thisData = ($chipCount0>0) ? $mean0/$chipCount0 : $noDataString;
		    $thisRefData = ($chipCount0>0) ? $meanRef0/$chipCount0 : $noDataString;
		    }
		elsif($chipIndex==4) {
		    $thisData = ($chipCount1>0) ? $mean1/$chipCount1 : $noDataString;
		    $thisRefData = ($chipCount1>0) ? $meanRef1/$chipCount1 : $noDataString;
		    }

		if($chipCount>0) {$cData /= $chipCount;}
		}

             else {
	        my ($chipName,$dataStar) = split(/:/,$chipdata[$chipIndex-5]);
		my @datas = split(/\*/,$dataStar);
		$thisData = $datas[$dataPointer];
		my ($cname,$cdata) = split(/:/,$cfactordata[$chipIndex-5]);
		my @reference_datas=split(/\*/,$refdata[$chipIndex-5]);
		if($doComparison) {
		  $thisRefData = ($scaleReferenceData) ? $reference_datas[$dataPointer] * $cdata : $reference_datas[$dataPointer];
		  }
		else {$thisRefData=0.;}
		$cData=$cdata;
		}
	  if($thisData eq "nan" || $thisData==0) {$thisData=$noDataString;}
          if($thisData ne $noDataString) {
	     if($doComparison) {
#	       printf "doing comparison of $thisData - $thisRefData <br>\n";
	       if($thisRefData!=0.) {$thisData-=$thisRefData;}
	       else {$thisData=$noDataString;}
	       }
	     }
          if($thisData ne $noDataString) {$thisData = ($isOccupancyData==1) ? sprintf("%.2E",$thisData) : sprintf("%.2f",$thisData);}

          $dataMap{$sn}=$thisData;
	  if(exists $daqInfo{$sn}) {$moduleCount++;}
	  $cData = sprintf("%.3f",$cData);
	  $cMap{$sn}=$cData;

	  }
        }
     close(IN) || printError("Cant close datafile: $!");
   }




sub readDaqTable() {

      my ($this_index) = @_;
      my $lookup = "$lookupDir$config{$dataSet}{MOUNTS}";


      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");
      
      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
      my $sctType = $config{$this_index}{TYPE};
      if(exists $addedModules{$sctType}) {
         for my $module (keys %{ $addedModules{$sctType} } ) {
#	     print "Adding extra module $module=$addedModules{$sctType}{$module}\n";
	     push(@lines,"$module=$addedModules{$sctType}{$module}");
	     }
         }

 #     push(@lines,"20220380200139=Barrel 3, Row 27, Position 6");

      foreach my $line (@lines) {
        if($line=~/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            $barrelCount[$2-3]++;
            my $position = $4;
            $position+=6;
            if($position>=7) {$position--;}
            my $newpos = ($position + ($3 * 12))<<4;
            my $index = $2 - 3 + $newpos;
            $daqMapping{$index} = $1;

            $daqInfoExists++;
            }
          }
	  
      foreach my $rmodule (keys %replacedModules) {
	  my $type=$config{$this_index}{TYPE};
          if((exists $replacedModules{$rmodule}{TYPE}) && $replacedModules{$rmodule}{TYPE}==$type) {
	     $daqInfo{$replacedModules{$rmodule}{LAST}} = $daqInfo{$rmodule};
#	     print "$rmodule now in same location as $replacedModules{$rmodule}{LAST} from Run $replacedModules{$rmodule}{FROMRUN}\n";
	     }
          }

#      close(IN) || printError("Cant close $lookup: $!");
      }


sub printBarrelFile {

my $barrel=$_[0];


my $rows = $barrelRows[$barrel];

#my $thisTitle = "Latest $testNames[$testIndex] $parameterNames[$parameterIndex] Data for Barrel $barrel";

print qq!

<h3>SctRodDaq Data Display for $config{$dataSet}{TITLE}</h3>
This page is currently displaying the most recent <b>$testIndexNames[$testIndex]</b> data for <b>$moduleCount</b> modules.<br><br>
Data for this webscript is collated automatically over night, so new data may not be visible for up to 24 hours.<br>
Also note that modules which have not been shipped to Oxford in the SCT database will not appear, even if they are physically on the barrel.<br>
For further details of tests etc, click on the run number in the <a href="$logScript">SctRodDaq Logbook</a><br>
Please note that row numbering on this display refers to the DAQ row numbering, which starts from 0.
LMT numbering starts from 1, so row 0 is equivalent to LMT01.

<br><br>

<form method="POST">
<table border=0 cellspacing=1 cellpadding=0>
<col width=250>
<col width=190>
<col width=180>
<col width=180>
<tr><td colspan=4><hr></td></tr>
<tr><td colspan=4 align=left><b>Display Update Options</b></td></tr>
<tr><td colspan=4><br></td></tr>
<tr><td colspan=4 align=left>DataSet:&nbsp;&nbsp;<SELECT NAME="dataSet">
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
!;
my $ndsets = (keys %config);
for (my $i=0;$i<$ndsets;$i++) {
    my $selectflag = ($dataSet == $i) ? "SELECTED" : "";
    my $title = ($i==0) ? "Current Data ($config{$i}{TITLE})" : $config{$i}{TITLE};
    print "<option value=$i $selectflag>$title</option>\n";
    }
print qq!
</SELECT>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<INPUT TYPE="SUBMIT" NAME="showDataSetInfo" VALUE="View DataSet Information">
</td>
</td></tr>
<tr><td colspan=4><br></td></tr>
<tr><td colspan=4 align=left>
!;
my $noTests = @testIndexNames;
printf "Test Type:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
for(my $i=0;$i<$noTests;$i++) {
 if($i>0) {printf "&nbsp;&nbsp;&nbsp;&nbsp;-&nbsp;&nbsp;&nbsp;&nbsp;";}
 printf "<a href=$scriptname\?testIndex=$i\&parameterIndex=0\&dataSet=$dataSet>$testIndexNames[$i]</a>";
 }
print qq!
</td></tr>
<tr><td colspan=4><br></td></tr>
<tr>

<td align=left nowrap>Parameter:&nbsp;&nbsp;<SELECT NAME="parameterIndex">
!;
  my $count=0;
  foreach (@paramlist) {
    my $selectflag = ($parameterIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }
print qq!
</SELECT></td>

<td align=left nowrap>Chip (if app):&nbsp;&nbsp;<SELECT NAME="chipIndex">
!;
  $count=0;
  foreach (@chipNames) {
    my $selectflag = ($chipIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }

print qq!
  </SELECT></td>
  <td align=left nowrap>Cell Label:&nbsp;&nbsp;<SELECT NAME="cellfillIndex">
!;
    $count=0;
    foreach (@cellfills) {
      my $selectflag = ($cellfillIndex==$count) ? "SELECTED" : "";
      print "<option value=$count $selectflag>$_</option>\n";
      $count++;
    }
print qq!
</SELECT></td>
<td align=left nowrap><INPUT TYPE="SUBMIT" NAME="viewplots" VALUE="Refresh Display"></td></tr>
<tr><td colspan=4><br></td></tr>
<tr><td colspan=4><hr></td></tr>

</table>
<input type="HIDDEN" name="parameterIndex" VALUE="$parameterIndex">
<input type="HIDDEN" name="testIndex" VALUE="$testIndex">
</form>
<br>
!;


print qq!
<table border=0 cellspacing=5 cellpadding=0>
<col width=150>
<col width=300>
<col width=400>
<tr><td align=right nowrap>Displayed Data Set:</td><td valign=top align=left nowrap><font color=navy>$config{$dataSet}{TITLE} </font></td>
<td rowspan=3 valign=center align=left>
!;
&printColorScale();

printf "</td></tr>";
printf "<tr><td align=right nowrap>Displayed Test:</td><td valign=top align=left nowrap><font color=navy>$testIndexNames[$testIndex] </font></td></tr>";
printf "<tr><td align=right nowrap>Displayed Colour scale:</td><td align=left nowrap> <font color=navy>$paramlist[$parameterIndex]";
if($dataPointer<=1) {printf " ($chipNames[$chipIndex])";}
printf "</font></td></tr>";
printf "<tr><td align=right nowrap>Displayed Cell Label:</td><td align=left nowrap><font color=navy>$cellfills[$cellfillIndex] </font></td></tr>";

printf "</table>";

print qq!
<br>
Hints:<br>
&gt;&nbsp; To view a detailed test report, click on the data value.<br>
&gt;&nbsp; To view all tests on a module, select Cell Label to 'serialNo' and click on the serial number.<br>
<br>

<table border=0 cellspacing=4 cellpadding=0>


<tr>
 <td nowrap width="50" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="10" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
</tr>

<tr><td nowrap></td><td nowrap colspan=13 align=center><b>Module Position</b></td></tr>

<tr>
<td nowrap></td>
<td nowrap align=center><b>-6</b></td>
<td nowrap align=center><b>-5</b></td>
<td nowrap align=center><b>-4</b></td>
<td nowrap align=center><b>-3</b></td>
<td nowrap align=center><b>-2</b></td>
<td nowrap align=center><b>-1</b></td>
<td nowrap></td>
<td nowrap align=center><b>1</b></td>
<td nowrap align=center><b>2</b></td>
<td nowrap align=center><b>3</b></td>
<td nowrap align=center><b>4</b></td>
<td nowrap align=center><b>5</b></td>
<td nowrap align=center><b>6</b></td>
</tr>
!;

for (my $row=0;$row<$rows; $row++) {
 print "<tr><td nowrap align=right><b>Row $row: </b></td>\n";
 for (my $pos=0;$pos<12;$pos++) {
   my $posindex = ($pos + ($row * 12))<<4;
   my $index = $barrel + $posindex;
   my $content = "";
   my $cellColor = "#eeeeff";
   if(exists $daqMapping{$index}) {
     my $sn = $daqMapping{$index};

     # Check for replaced modules.
     # We might need $index to point to the older (replaced) module.
     # First check if this is module has replaced another
     if(exists $replacedModules{$sn}{LAST}) {
# if the datalinks exist, it means there is data from the new module
       if((exists $datalinks{$sn}) && $datalinks{$sn}=~/.*runno=(\d+).*/) {
# if the run number is >= FROMRUN, the use the new module, otherwise use the old module
         if($1 < $replacedModules{$sn}{FROMRUN}) {$sn = $replacedModules{$sn}{LAST};}
	 }
	 # if no data exists for the new module, then assume this inde points to the replaced module
       else {$sn = $replacedModules{$sn}{LAST};}
       }
     if(exists $dataMap{$sn}) {
        if($cellfillIndex==0) {$content="<a href=${scriptname}\?getTest=1\&$datalinks{$sn} class=\"white\">$dataMap{$sn}</a>";}
	elsif($cellfillIndex==1) {$content="<a href=${scriptname}\?getList=1\&$datalinks{$sn} class=\"white\">$sn</a>";}
	elsif($cellfillIndex==2) {$content = (exists $cMap{$sn}) ? $cMap{$sn} : "Unavailable";}
	else {$content="";}
        $cellColor = &getCellColor($dataMap{$sn});
	}
 #     else {print "No dataMap for $sn \n";}
     }
 #  else {print "No daqMapping for $index \n";}

   print "<td nowrap bgcolor=\"$cellColor\" align=center>$content</td>";
   if($pos==5) {print "<td></td>";}
   }
 print "</tr>\n";
 }
 print "</table>\n";
 
 print qq!
 <br><br><b>Defective Channel Counts</b><br>
 !;
 if($totaldefectcount==-1) {
   print "<br>No defects present...<br>";
   return;
   }
 print qq!
 <table border=1 cellspacing=0 cellpadding=0>
 <col width=150>
 <col width=100>
 <tr><td><b>Defect Name</b></td><td><b>#Channels</b></td></tr>
 !;
 foreach my $defect (keys %defectCount) {
   print "<tr><td align=left nowrap>$defect</td><td align=left>$defectCount{$defect}</td></tr>\n";
   }
 print qq!
 <tr><td align=left nowrap><b>Total:</b></td><td align=left nowrap><b>$totaldefectcount</b></td></tr>
 </table>
 !;




}

sub getCellColor() {


  my ($data) = @_;
  
  if($data eq $noDataString) {return "#f1f1f1";}

  my $color = "000000";
  my $bluebit=0;
  my $redbit=0;
  my $greenbit=0;
  
  my $paramKey = $paramlist[$parameterIndex];
#  if(!(exists colorHashC0{$paramKey})) {printError("No color scale");}

  if($data < $colorHashC0{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
    return $color;
    }
    
  if($data > $colorHashC3{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
    return $color;
    }


  my $value0 = $colorHashC0{$paramKey}[0];
  my $value1 = $colorHashC1{$paramKey}[0];
  my $value2 = $colorHashC2{$paramKey}[0];
  my $value3 = $colorHashC3{$paramKey}[0];

  if($data<$value1) {
    my $diff = $data - $value0;
    my $max = $value1 - $value0;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC0{$paramKey}[1];
    my $greenbit0 = $colorHashC0{$paramKey}[2];
    my $bluebit0 = $colorHashC0{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>$value2) {
    my $diff = $data - $value2;
    my $max = $value3 - $value2;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC2{$paramKey}[1];
    my $greenbit0 = $colorHashC2{$paramKey}[2];
    my $bluebit0 = $colorHashC2{$paramKey}[3];
    
    my $redbit1 = $colorHashC3{$paramKey}[1];
    my $greenbit1 = $colorHashC3{$paramKey}[2];
    my $bluebit1 = $colorHashC3{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>=$value1 && $data<=$value2) {
    my $diff = $data - $value1;
    my $max = $value2 - $value1;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC1{$paramKey}[1];
    my $greenbit0 = $colorHashC1{$paramKey}[2];
    my $bluebit0 = $colorHashC1{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }


  return "#ffffff";
  }

sub printColorScale {

my $paramKey = $paramlist[$parameterIndex];
my $C0color = sprintf("#%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
my $C1color = sprintf("#%02x%02x%02x",$colorHashC1{$paramKey}[1],$colorHashC1{$paramKey}[2],$colorHashC1{$paramKey}[3]);
my $C2color = sprintf("#%02x%02x%02x",$colorHashC2{$paramKey}[1],$colorHashC2{$paramKey}[2],$colorHashC2{$paramKey}[3]);
my $C3color = sprintf("#%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
my $label0 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC0{$paramKey}[0]) : $colorHashC0{$paramKey}[0];
my $label1 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC1{$paramKey}[0]) : $colorHashC1{$paramKey}[0];
my $label2 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC2{$paramKey}[0]) : $colorHashC2{$paramKey}[0];
my $label3 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC3{$paramKey}[0]) : $colorHashC3{$paramKey}[0];

print qq!
<table border=0 cellspacing=0 cellpadding=0>
<tr><td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label0}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label1}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label2}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label3}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
<tr><td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td></tr>
<tr>


<td align=center bgcolor="$C0color">&nbsp;</td>
<td align=center bgcolor="$C1color">&nbsp;</td>
<td align=center bgcolor="$C2color">&nbsp;</td>
<td align=center bgcolor="$C3color">&nbsp;</td></tr>
</table>
!;
}

sub printPageHeader() {
my ($title) = @_;
print qq!
<html>
<head>
<title>$title</title>
<link rel=stylesheet HREF="http://www.hep.phy.cam.ac.uk/~robinson/styles/davestyles2.css" type="text/css">
!;
print "<\!-- \@author Dave Robinson 21/10/2004 -->";
print qq!
</head>
<body>
!;
}

sub showTestData() {
 my $runno = param("runno") ? param("runno") : "";
 my $scanno = param("scanno") ? param("scanno") : "0";
 my $testNameHeader = param("test") ? param("test") : "";
 my $sn = param("sn") ? param("sn") : "";
 my $dataSet = param("dataSet") ? param("dataSet") : "0";

 if($runno eq "" || $testNameHeader eq "" || $sn eq "") {printError("invalid arguments to script");}

 my $linkback = "\&gt;&nbsp;&nbsp;<a href=$scriptname\?testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>";
 $linkback .= "\&gt;&nbsp;&nbsp;<a href=$scriptname\?getList=1\&sn=$sn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to list ALL tests for this module.<br><br>";

 parseSummaryData("$rootDirectory$config{$dataSet}{DATADIR}/summaries/",$runno,$scanno,"${testNameHeader}TestResult",$sn,$linkback);
 }

sub showTestList() {
 my $sn = param("sn") ? param("sn") : "";

 if($sn eq "") {printError("invalid arguments to script");}
 
 my $linkback = "\&gt;&nbsp;&nbsp;<a href=$scriptname\?testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>";

 printModuleTestList("$rootDirectory$config{$dataSet}{DATADIR}/summaries/",$dataSet,$sn,$scriptname,$linkback);

  }
  
sub showDataSetInfo() {
my $rmsg = ($config{$dataSet}{RUNEX} eq "") ? "No runs have been excluded when compiling this dataset." : "Run(s) $config{$dataSet}{RUNEX} were excluded when compiling this dataset.";
my $runmsg = ($config{$dataSet}{RUNHI} ==99999) ? "This dataset comprises the most recent data taken since Run <b>$config{$dataSet}{RUNLO}</b>" : "This dataset comprises the most recent data from Run <b>$config{$dataSet}{RUNLO}</b> to Run <b>$config{$dataSet}{RUNHI}";
 print qq!
 <h3>$testIndexNames[$testIndex] Dataset Information for $config{$dataSet}{TITLE}</h3>
 $runmsg<br><br>
 $rmsg
 <br>
 <br>
 <table border=1>
 <col width=150>
 <col width=100>
 <col width=100>
 <col width=250>
 <tr><td>Serial Number</td><td>Run</td><td>Scan</td><td>Comments</td></tr>
 !;
 
 my $fname = "$lookupDir$testIndexNames[$testIndex]_wwwsummary_$dataSet.txt";

 open(IN,$fname) || printError("Cant open $fname: $!");
 flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
#RUN=1371SCAN=0SN=20220040200007NOISE=M0:1183.21;S1:1211.68;S2:1293.97;S3:1283.25;S4:1257.9;E5:1254.65;M8:1220.48;S9:1280.05;S10:1266.41;S11:1214.65;S12:1338.63;E13:1307.26CFACTORS=M0:1.00;S1:1.00;S2:1.00;S3:1.00;S4:1.00;E5:1.00;M8:1.00;S9:1.00;S10:1.00;S11:1.00;S12:1.00;E13:1.00
    while(<IN>) {
       if(/^DEFECT(.*)=(\d+)/) {$defectCount{$1}=$2;}
       if(/^TOTALDEFECTCOUNT=(\d+)/) {$totaldefectcount=$1;}
       if(/.*RUN=(\d+)SCAN=(\d+)SN=(\d{14})DATA=(.*)DCS=(.*)DEFECTS=(.*)CFACTORS=(.*)CONF=(.*)REF=(.*)/) {
            my $runno=$1;
	    my $scanno=$2;
	    my $sn=$3;
	    my $comment = "";
	    if($5 eq ".*.*.*.*.*.*.*.") {$comment = "No DCS info. ";}
	    if($7=~/.*1.00.*/) {$comment .= "Some or all data taken with cfactor=1.00 ";}
	    if($comment eq "") {$comment = "None";}
	    print "<tr><td>$sn</td><td>$runno</td><td>$scanno</td><td>$comment</td></tr>\n";
	    }
       }
    close(IN) || printError("Cant close $fname: $1");
    print qq!
    </table>
    <br>
    &gt;&nbsp;&nbsp;<a href=$scriptname\?testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>
    !;
}


 


